@@ -1,4 +1,4 @@
-# ClamTk, copyright (C) 2004-2014 Dave M
+# ClamTk, copyright (C) 2004-2015 Dave M
 #
 # This file is part of ClamTk (http://code.google.com/p/clamtk/).
 #
@@ -24,7 +24,7 @@ use Encode 'decode';
 
 sub get_TK_version {
     # Stick with %.2f format - 4.50 vice 4.5
-    return '5.11';
+    return '5.14';
 }
 
 sub get_path {
@@ -45,6 +45,16 @@ sub get_path {
     # Default personal clamtk directory
     $path->{ clamtk } = $path->{ directory } . '/.clamtk';
 
+    # Trash directory - main
+    $path->{ trash_dir } = $path->{ directory } . '/.local/share/Trash';
+
+    # Trash directory - where files are held
+    $path->{ trash_dir_files } = $path->{ trash_dir } . '/files';
+
+    # Trash directory - where associated files are held:
+    # e.g. trash.jpg.trashinfo
+    $path->{ trash_files_info } = $path->{ trash_dir } . '/info';
+
     # For storing quarantined files
     $path->{ viruses } = $path->{ clamtk } . '/viruses';
 
